"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _isId = _interopRequireDefault(require("./isId"));

var _generateId = _interopRequireDefault(require("./generateId"));

var _fieldsStorage = require("@commodo/fields-storage");

var _name = require("@commodo/name");

var _lambda = _interopRequireDefault(require("aws-sdk/clients/lambda"));

var _bson = require("bson");

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { (0, _defineProperty2.default)(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

class DbProxyClient {
  constructor({
    dbProxyFunctionName
  }) {
    (0, _defineProperty2.default)(this, "dbProxyFunctionName", void 0);
    this.dbProxyFunctionName = dbProxyFunctionName;
  }

  async runOperation(requestPayload) {
    const Lambda = new _lambda.default({
      region: process.env.AWS_REGION
    });
    const {
      Payload
    } = await Lambda.invoke({
      FunctionName: this.dbProxyFunctionName,
      Payload: JSON.stringify({
        body: _bson.EJSON.stringify(requestPayload)
      })
    }).promise();
    let parsedPayload;

    try {
      parsedPayload = JSON.parse(Payload);
    } catch (e) {
      throw new Error("Could not JSON.parse DB Proxy's response.");
    }

    if (parsedPayload.error) {
      throw new Error(`${parsedPayload.error.name}: ${parsedPayload.error.message}`);
    }

    if (!parsedPayload.response) {
      throw new Error(`Missing "response" key in received DB Proxy's response.`);
    }

    const {
      result
    } = _bson.EJSON.parse(parsedPayload.response);

    return result;
  }

}

class DbProxyDriver {
  constructor({
    dbProxyFunctionName = process.env.DB_PROXY_FUNCTION_NAME
  } = {}) {
    (0, _defineProperty2.default)(this, "client", void 0);
    this.client = new DbProxyClient({
      dbProxyFunctionName
    });
  } // eslint-disable-next-line


  async save({
    model,
    isCreate
  }) {
    return isCreate ? this.create({
      model
    }) : this.update({
      model
    });
  }

  async create({
    model
  }) {
    if (!model.id) {
      model.id = (0, _generateId.default)();
    }

    const data = await model.toStorage();

    try {
      await this.client.runOperation({
        collection: this.getCollectionName(model),
        operation: ["insertOne", data]
      });
      return true;
    } catch (e) {
      model.id && model.getField("id").reset();
      throw e;
    }
  }

  async update({
    model
  }) {
    const data = await model.toStorage();
    await this.client.runOperation({
      collection: this.getCollectionName(model),
      operation: ["updateOne", {
        id: model.id
      }, {
        $set: data
      }]
    });
    return true;
  } // eslint-disable-next-line


  async delete({
    model
  }) {
    await this.client.runOperation({
      collection: this.getCollectionName(model),
      operation: ["deleteOne", {
        id: model.id
      }]
    });
    return true;
  }

  async find({
    model,
    options
  }) {
    const clonedOptions = _objectSpread({
      limit: 10,
      offset: 0
    }, options);

    DbProxyDriver.__preparePerPageOption(clonedOptions);

    DbProxyDriver.__preparePageOption(clonedOptions);

    DbProxyDriver.__prepareSearchOption(clonedOptions);

    const $facet = {
      results: [{
        $skip: clonedOptions.offset
      }, {
        $limit: clonedOptions.limit
      }]
    };

    if (clonedOptions.sort) {
      $facet.results.unshift({
        $sort: clonedOptions.sort
      });
    }

    if (options.meta !== false) {
      $facet.totalCount = [{
        $count: "value"
      }];
    }

    const pipeline = [{
      $match: clonedOptions.query
    }, {
      $facet
    }];
    const [results = {}] = await this.client.runOperation({
      collection: this.getCollectionName(model),
      operation: ["aggregate", pipeline]
    });

    if (!Array.isArray(results.results)) {
      results.results = [];
    }

    if (!Array.isArray(results.totalCount)) {
      results.totalCount = [];
    }

    return [results.results, (0, _fieldsStorage.createPaginationMeta)({
      totalCount: results.totalCount[0] ? results.totalCount[0].value : 0,
      page: options.page,
      perPage: options.perPage
    })];
  }

  async findOne({
    model,
    options
  }) {
    const clonedOptions = _objectSpread({}, options);

    DbProxyDriver.__preparePerPageOption(clonedOptions);

    DbProxyDriver.__preparePageOption(clonedOptions);

    DbProxyDriver.__prepareSearchOption(clonedOptions); // Get first documents from cursor using each


    const results = await this.client.runOperation({
      collection: this.getCollectionName(model),
      operation: ["find", clonedOptions.query, {
        limit: 1,
        sort: clonedOptions.sort
      }]
    });
    return results[0];
  }

  async count({
    model,
    options
  }) {
    const clonedOptions = _objectSpread({}, options);

    DbProxyDriver.__prepareSearchOption(clonedOptions); // Get first documents from cursor using each


    return await this.client.runOperation({
      collection: this.getCollectionName(model),
      operation: ["count", clonedOptions.query]
    });
  }

  isId(value) {
    return (0, _isId.default)(value);
  }

  getCollectionName(model) {
    return (0, _name.getName)(model);
  }

  getClient() {
    return this.client;
  }

  static __preparePerPageOption(options) {
    if ("perPage" in options) {
      options.limit = options.perPage;
      delete options.perPage;
    }
  }

  static __preparePageOption(options) {
    if ("page" in options) {
      options.offset = options.limit * (options.page - 1);
      delete options.page;
    }
  }

  static __prepareSearchOption(options) {
    // Here we handle search (if passed) - we transform received arguments into linked LIKE statements.
    if (options.search && options.search.query) {
      const {
        query,
        operator,
        fields
      } = options.search;
      const searches = [];
      fields.forEach(field => {
        searches.push({
          [field]: {
            $regex: `.*${query}.*`,
            $options: "i"
          }
        });
      });
      const search = {
        [operator === "and" ? "$and" : "$or"]: searches
      };

      if (options.query instanceof Object) {
        options.query = {
          $and: [search, options.query]
        };
      } else {
        options.query = search;
      }

      delete options.search;
    }
  }

}

var _default = DbProxyDriver;
exports.default = _default;
//# sourceMappingURL=DbProxyDriver.js.map